from django.contrib import admin
from .forms import CategoryForm, CategorySuperuserForm

class CategoryAdmin(admin.ModelAdmin):
    list_display = ['title', 'sub_categories', 'active']

    def get_form(self, request, obj=None, **kwargs):
        if request.user.is_superuser:
            kwargs['form'] = CategorySuperuserForm
        else:
            kwargs['form'] = CategoryForm

        return super(CategoryAdmin, self).get_form(request, obj, **kwargs)

    def get_queryset(self, request):
        queryset = super(CategoryAdmin, self).get_queryset(request)

        if request.path_info == '/admin/%s/%s/' % (
                self.model._meta.app_label.lower(),
                self.model._meta.object_name.lower()):

            queryset = queryset.filter(parent__isnull=True)

        return queryset

    class Meta:
        abstract = True



from django import forms


class CategoryForm(forms.ModelForm):

    def __init__(self, *args, **kwargs):
        super(CategoryForm, self).__init__(*args, **kwargs)
        try:
            exclude_ids = [kwargs['instance'].id]
            exclude_ids += self.child_ids(kwargs['instance'])
            qs = self._meta.model.objects.exclude(id__in=exclude_ids)
        except:
            qs = self._meta.model.objects.all()

        self.fields['parent'] = forms.ModelChoiceField(
            queryset=qs,
            required=False)

    def child_ids(self, instance):
        child_id_list = []
        items = type(instance).objects.filter(parent=instance)
        for item in items:
            child_id_list.append(item.id)
            child_id_list += self.child_ids(item)

        return child_id_list

    class Meta:
        exclude = ('active',)


class CategorySuperuserForm(CategoryForm):
    class Meta:
        fields = '__all__'



from django.db import models
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _


class Category(models.Model):
    title = models.CharField(max_length=250)
    parent = models.ForeignKey('self', blank=True, null=True)
    active = models.BooleanField(default=True)

    def __unicode__(self):
        if self.parent:
            return '%s > %s' % (self.parent.title, self.title)
        else:
            return self.title
        return '%s_%s' % (self._meta.app_label, self._meta.object_name)

    def sub_categories(self):
        childs = type(self).objects.filter(parent=self)
        output = '<ul>'
        for child in childs:
            subchilds = child.sub_categories()
            url = reverse('admin:%s_%s_change' % (
                self._meta.app_label.lower(),
                self._meta.object_name.lower()), args=(child.id,))
            output = '%s<li><a href=\'%s\'>%s</a>%s</li>' % (
                output, url, child.title, subchilds)

        output = '%s</ul>' % output
        return output

    sub_categories.allow_tags = True

    class Meta:
        abstract = True
        verbose_name = 'Category'
        verbose_name_plural = 'Categories'



from setuptools import setup

setup(
    name='django_categorytree',
    version='0.1.1',
    description='A django app with an abstract model which helps \
to define tree style categories with unlimited sub levels',
    url='https://github.com/payamnj/django-categorytree',
    author='Payam Najafizadeh',
    author_email='payam.nj@gmail.com',
    license='New BSD',
    packages=['categorytree', ],
    classifiers=[
        'Programming Language :: Python',
        'Operating System :: OS Independent',
        'Environment :: Web Environment',
        'Framework :: Django'],
    requires=['django(>=1.8)'],)
